@@ -26,3 +26,13 @@ require_relative "io/version"
 
 require_relative "io/endpoint"
 require_relative "io/endpoint/each"
+
+module Async
+	module IO
+		@file_descriptor_limit = nil
+		
+		def self.file_descriptor_limit
+			@file_descriptor_limit ||= Integer(`ulimit -n`) rescue nil
+		end
+	end
+end
