@@ -30,7 +30,13 @@ module Async
 			end
 			
 			def to_s
-				"\#<#{self.class} #{@specification.inspect}>"
+				nodename, service, family, socktype, protocol, flags = @specification
+				
+				"\#<#{self.class} name=#{nodename.inspect} service=#{service.inspect} family=#{family.inspect} type=#{socktype.inspect} protocol=#{protocol.inspect} flags=#{flags.inspect}>"
+			end
+			
+			def address
+				@specification
 			end
 			
 			def hostname
@@ -41,14 +47,24 @@ module Async
 			# @yield [Socket] the socket which is being connected, may be invoked more than once
 			# @return [Socket] the connected socket
 			# @raise if no connection could complete successfully
-			def connect(&block)
+			def connect
 				last_error = nil
 				
+				task = Task.current
+				
 				Addrinfo.foreach(*@specification) do |address|
 					begin
-						return Socket.connect(address, **@options, &block)
-					rescue
+						wrapper = Socket.connect(address, **@options, task: task)
+					rescue Errno::ECONNREFUSED, Errno::ENETUNREACH, Errno::EAGAIN
 						last_error = $!
+					else
+						return wrapper unless block_given?
+						
+						begin
+							return yield wrapper, task
+						ensure
+							wrapper.close
+						end
 					end
 				end
 				
@@ -75,13 +91,20 @@ module Async
 		end
 		
 		class Endpoint
-			# args: nodename, service, family, socktype, protocol, flags
+			# @param args nodename, service, family, socktype, protocol, flags. `socktype` will be set to Socket::SOCK_STREAM.
+			# @param options keyword arguments passed on to {HostEndpoint#initialize}
+			#
+			# @return [HostEndpoint]
 			def self.tcp(*args, **options)
 				args[3] = ::Socket::SOCK_STREAM
 				
 				HostEndpoint.new(args, **options)
 			end
 
+			# @param args nodename, service, family, socktype, protocol, flags. `socktype` will be set to Socket::SOCK_DGRAM.
+			# @param options keyword arguments passed on to {HostEndpoint#initialize}
+			#
+			# @return [HostEndpoint]
 			def self.udp(*args, **options)
 				args[3] = ::Socket::SOCK_DGRAM
 				
