@@ -18,22 +18,18 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-require_relative '../stream'
+require_relative 'generic'
 
 module Async
 	module IO
 		module Protocol
-			class Line
+			class Line < Generic
 				def initialize(stream, eol = $/)
-					@stream = stream
+					super(stream)
+					
 					@eol = eol
 				end
 				
-				def close
-					@stream.close
-				end
-				
-				attr :stream
 				attr :eol
 				
 				def write_lines(*args)
