@@ -18,7 +18,7 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-require_relative 'endpoint'
+require_relative 'host_endpoint'
 require_relative 'ssl_socket'
 
 module Async
@@ -40,6 +40,10 @@ module Async
 				"\#<#{self.class} #{@endpoint}>"
 			end
 			
+			def address
+				@endpoint.address
+			end
+			
 			def hostname
 				@options[:hostname] || @endpoint.hostname
 			end
@@ -90,7 +94,7 @@ module Async
 				return to_enum unless block_given?
 				
 				@endpoint.each do |endpoint|
-					yield self.class.new(endpoint, @options)
+					yield self.class.new(endpoint, **@options)
 				end
 			end
 		end
@@ -99,6 +103,12 @@ module Async
 		SecureEndpoint = SSLEndpoint
 		
 		class Endpoint
+			# @param args
+			# @param ssl_context [OpenSSL::SSL::SSLContext, nil]
+			# @param hostname [String, nil]
+			# @param options keyword arguments passed through to {Endpoint.tcp}
+			#
+			# @return [SSLEndpoint]
 			def self.ssl(*args, ssl_context: nil, hostname: nil, **options)
 				SSLEndpoint.new(self.tcp(*args, **options), ssl_context: ssl_context, hostname: hostname)
 			end
