@@ -19,6 +19,7 @@
 # THE SOFTWARE.
 
 require 'async/io/socket'
+require 'async/io/address'
 
 require_relative 'generic_examples'
 
@@ -37,7 +38,7 @@ RSpec.describe Async::IO::Socket do
 		it "should fail to connect if no listening server" do
 			expect do
 				Async::IO::Socket.connect(address)
-			end.to raise_error(Errno::ECONNREFUSED)
+			end.to raise_exception(Errno::ECONNREFUSED)
 		end
 		
 		it "should close the socket when interrupted by a timeout" do
@@ -47,7 +48,7 @@ RSpec.describe Async::IO::Socket do
 			expect(wrapper).to receive(:close)
 			expect do
 				Async::IO::Socket.connect(address)
-			end.to raise_error(Async::TimeoutError)
+			end.to raise_exception(Async::TimeoutError)
 		end
 	end
 	
@@ -57,7 +58,7 @@ RSpec.describe Async::IO::Socket do
 			
 			expect do
 				Async::IO::Socket.bind(address)
-			end.to raise_error(Errno::EACCES)
+			end.to raise_exception(Errno::EACCES)
 		end
 		
 		it "can bind to port 0" do
@@ -71,6 +72,34 @@ RSpec.describe Async::IO::Socket do
 		end
 	end
 	
+	describe '#sync' do
+		it "should set TCP_NODELAY" do
+			address = Async::IO::Address.tcp('127.0.0.1', 0)
+			
+			socket = Async::IO::Socket.wrap(::Socket::AF_INET, ::Socket::SOCK_STREAM, ::Socket::IPPROTO_TCP)
+			
+			socket.sync = true
+			expect(socket.getsockopt(Socket::IPPROTO_TCP, Socket::TCP_NODELAY).bool).to be true
+			
+			socket.close
+		end
+	end
+	
+	describe '#timeout' do
+		subject{described_class.pair(:UNIX, :STREAM, 0)}
+		
+		it "should timeout while waiting to receive data" do
+			s1, s2 = *subject
+			
+			s2.timeout = 1
+			
+			expect{s2.recv(32)}.to raise_exception(Async::TimeoutError, "execution expired")
+			
+			s1.close
+			s2.close
+		end
+	end
+	
 	describe '.pair' do
 		subject{described_class.pair(:UNIX, :STREAM, 0)}
 		
@@ -96,6 +125,21 @@ RSpec.describe Async::IO::Socket do
 			s2.close
 		end
 	end
+	
+	context '.pipe' do
+		let(:sockets) do
+			@sockets = described_class.pair(Socket::AF_UNIX, Socket::SOCK_STREAM)
+		end
+		
+		after do
+			@sockets&.each(&:close)
+		end
+		
+		let(:io) {sockets.first}
+		subject {sockets.last}
+		
+		it_should_behave_like Async::IO
+	end
 end
 
 RSpec.describe Async::IO::IPSocket do
