@@ -22,7 +22,7 @@ require 'async/io/tcp_socket'
 
 require_relative 'generic_examples'
 
-RSpec.describe Async::IO::TCPSocket do
+RSpec.describe Async::IO::TCPSocket, timeout: 1 do
 	include_context Async::RSpec::Reactor
 	
 	it_should_behave_like Async::IO::Generic
@@ -33,24 +33,46 @@ RSpec.describe Async::IO::TCPSocket do
 	
 	describe Async::IO::TCPServer do
 		it_should_behave_like Async::IO::Generic
-		
-		it "should start server and send data" do
-			# Accept a single incoming connection and then finish.
-			server_task = reactor.async do |task|
+	end
+	
+	describe Async::IO::TCPServer do
+		let!(:server_task) do
+			reactor.async do |task|
 				server = Async::IO::TCPServer.new("localhost", 6788)
 				
 				peer, address = server.accept
 				
 				data = peer.gets
 				peer.puts(data)
+				peer.flush
 				
 				peer.close
 				server.close
 			end
+		end
+		
+		let(:client) {Async::IO::TCPSocket.new("localhost", 6788)}
+		
+		it "can read into output buffer" do
+			client.puts("Hello World")
+			client.flush
 			
-			client = Async::IO::TCPSocket.new("localhost", 6788)
+			buffer = String.new
+			# 20 is bigger than echo response...
+			data = client.read(20, buffer)
 			
+			expect(buffer).to_not be_empty
+			expect(buffer).to be == data
+			
+			client.close
+			server_task.wait
+		end
+		
+		it "should start server and send data" do
+			# Accept a single incoming connection and then finish.
 			client.puts(data)
+			client.flush
+			
 			expect(client.gets).to be == data
 			
 			client.close
