@@ -49,7 +49,8 @@ module Wrap
 	end
 end
 
-RSpec.describe Async::IO::TCPSocket do
+# TruffleRuby uses Socket.tcp in Net::HTTP.get_response, not TCPSocket.open
+RSpec.describe Async::IO::TCPSocket, if: RUBY_ENGINE != "truffleruby" do
 	describe "inside reactor" do
 		include_context Async::RSpec::Reactor
 		
@@ -62,7 +63,7 @@ RSpec.describe Async::IO::TCPSocket do
 			
 			expect do
 				Net::HTTP.get_response('www.google.com', '/')
-			end.to_not raise_error
+			end.to_not raise_exception
 		end
 	end
 	
@@ -70,7 +71,7 @@ RSpec.describe Async::IO::TCPSocket do
 		it "should fetch page" do
 			expect do
 				Net::HTTP.get_response('www.google.com', '/')
-			end.to_not raise_error
+			end.to_not raise_exception
 		end
 	end
 end
